function [next_state_num, act, Qt] = sw_move_state(w12, b2, w23, b3, ...
    HiddenFunc, OutFunc, epsilon, randdata, current_state_num, maze_i, maze_j)
% ACTION : Get action, next state, and Q values.
%
% Syntax : [next_state_num, act, Qt] = SW_MOVE_STATE(w12, b2, w23, b3, ...
%            HiddenFunc, OutFunc, epsilon, randdata, current_state_num, maze_i, maze_j);
%
% Input  :  w12    (Required) : Weight (Between input layer and hidden layer)
%                               (9xNum. hidden units  Matrix)
%           b2     (Required) : Baias of hidden layer.
%                               (Num. hidden units x 1  Vector)
%           w23    (Required) : Weight (Between hidden layer and output layer)
%                               (Num. hidden units x 5  Matrix)
%           b3     (Required) : Baias of output layer.
%                               (4 x 1  Vector)
%           HiddenFunc (Required) : Definition of hidden function.
%                               (Char : 'Sigmoid')
%           OutFunc    (Required) : Definition of output function.
%                                 : (Char : 'SoftMax')
%           epsilon    (Required) : Epsilon. (Scalar : 0 to MaxEpisode)
%           randdata   (Required) : Random data (Scalar : 0 to MaxEpisode)
%           satate_num (Required) : State number (Scalar : 1 to Size of Maze (Ex. 3x3))
%           maze_i     (Required) : Size of Mase (i(y) axis : c)
%           maze_j     (Required) : Size of Mase (j(x) axis : )
%
% Output :  new_satate_num : Next state number (Scalar : 1 to Size of Maze (Ex. 3x3))
%           act : Action. (Scalar : 1 to 4)
%           Qt : Output of neural network. (4x1 Vector)
%
% Requirements : None.
%
% See also SW_Q_LEARNING.    

    % Create One hot input from state
    st = zeros(maze_i*maze_j,1);       % Create One hot input from state
    st(current_state_num) = 1;

    X = st;                             % Input (State Number)
    
    z2 = w12.' * X + b2;                % Weighted sum of the hidden layers z2
    if(HiddenFunc == "sigmoid")
        a2 = 1./(1+exp(-z2));           % Output a2 of the hidden layer (activation function: sigmoid function)
    end
    
    z3 = w23.' * a2 + b3;               % Weighted sum of the output layers z3
    if(OutFunc == 'Linear')
        a3 = z3;      % Output of output layer a3 (activation function: lenear function)
    end
    Qt = a3;                            % Store the output as Qt
    
    % Check the value
    % fprintf('\nQt =');
    % fprintf('%f ',Qt);
    % fprintf('\n');
    
    % Determine the action (-greedy algorithm)
    if epsilon <= randdata              % If random data is smaller than epsilon, act randomly.
        max_value = max(Qt);            % Use maximum Q-value to determine action.
        for j = 1 : 4
            if max_value == Qt(j)
                act = j ;               % The index of the maximum value is the action value
            end
        end
    else
        % Choose one of four actions (right, up, left, down) at 25 percent
        fprintf('RA'); % Display randum action.
        random_value = rand;
        if random_value <= 0.25
            act = 1 ; % right
        elseif random_value <= 0.5
            act = 2 ; % up
        elseif random_value <= 0.75
            act = 3 ; % left
        else
            act = 4 ; % down
        end
    end
    
    % Create a maze board that is moved to the destination.
    if act == 1            % right
        next_state_num = current_state_num + 1;
        % If the action results in going outside the board frame, the player stays there.
        if next_state_num > maze_i*maze_j || rem(next_state_num, maze_j) == 1
            next_state_num = current_state_num;
        end        
    elseif act == 2        % up
        next_state_num = current_state_num - maze_j;
        % If the action results in going outside the board frame, the player stays there.
        if next_state_num <= 0
            next_state_num = current_state_num;
        end
    elseif act == 3        % left
        next_state_num = current_state_num - 1;
        % If the action results in going outside the board frame, the player stays there.
        if next_state_num <= 0 || rem(next_state_num, maze_j) == 0
            next_state_num = current_state_num;
        end
    else                   % down
        next_state_num = current_state_num + maze_j;
        % If the action results in going outside the board frame, the player stays there.
        if next_state_num > maze_i*maze_j
            next_state_num = current_state_num;
        end
    end        
end